//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Programs.h"
#include "App_ProgramDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Files.h"
#include "Moa_ShellFiles.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_programs {
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramEntry::EnsureValidPath(const AnsiString& APath)
{
	AnsiString LPath = APath;
	try {
		if(!TPathUtilities::IsAbsolute(LPath)) {
			throw EProgramEntryError(AnsiString().sprintf(
				"The program cannot accept the path \"%d\" because it is not in absolute form.",
				APath.c_str()));
		}
		LPath = ExpandFileName(LPath);
		LPath = TFileUtilities::GetLongPath(LPath);
	}
	catch(EFileError& LException) {
		throw EProgramEntryError(AnsiString().sprintf(
			"The program cannot accept the path \"%s\" because its long form is unavailable.",
			LPath.c_str()));
	}
	return LPath;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Initialize()
{
	try {
		FIcon = new TIcon();
		UpdateIcon();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Uninitialize()
{
	delete FIcon;
	FIcon = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::UpdateIcon()
{
	try {
		if(IsDefault()) {
			TShellFileUtilities::GetIcon(
				TShellFileUtilities::FindIconByExtension("exe", isSmall),
				Icon,
				isSmall);
		}
		else {
			TShellFileUtilities::GetIcon(
				TShellFileUtilities::FindIconByPath(FFile, isSmall),
				Icon,
				isSmall);
		}
	}
	catch(EShellFileError& LException) {
		Icon->Handle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::SetFile(const AnsiString& AFile)
{
	AnsiString LFile;
	if(AFile.Length() > 0) {
		LFile = EnsureValidPath(AFile);
	}
	if(LFile == File) {
		return;
	}
	FFile = LFile;
	UpdateIcon();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::TProgramEntry() :
	TObject(),
	FIcon(NULL),
	FDisplayName(),
	FFile(),
	FDocumentPrefix(),
	FDocumentSuffix(),
	FWorkingDirectory(),
	FShowCommand(SW_NORMAL),
	FWaitForTermination(true)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::~TProgramEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Assign(TProgramEntry* ASource)
{
	if(ASource == this) {
		return;
	}
	FIcon->Assign(ASource->Icon);
	FDisplayName = ASource->FDisplayName;
	FFile = ASource->FFile;
	FDocumentPrefix = ASource->FDocumentPrefix;
	FDocumentSuffix = ASource->FDocumentSuffix;
	FWorkingDirectory = ASource->FWorkingDirectory;
	FShowCommand = ASource->FShowCommand;
	FWaitForTermination = ASource->FWaitForTermination;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Load(TMemIniFile* ASettings, const AnsiString& ASection)
{
	try {
		TAutoPointer<TProgramEntry> LInstance = new TProgramEntry();
		LInstance->DisplayName = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DisplayName",
			TAnsiStringUtilities::Quote(LInstance->DisplayName)));
		LInstance->File = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"File",
			TAnsiStringUtilities::Quote(LInstance->File)));
		LInstance->DocumentPrefix = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DocumentPrefix",
			TAnsiStringUtilities::Quote(LInstance->DocumentPrefix)));
		LInstance->DocumentSuffix = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DocumentSuffix",
			TAnsiStringUtilities::Quote(LInstance->DocumentSuffix)));
		LInstance->WorkingDirectory = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"WorkingDirectory",
			TAnsiStringUtilities::Quote(LInstance->WorkingDirectory)));
		LInstance->ShowCommand = ASettings->ReadInteger(
			ASection,
			"ShowCommand",
			LInstance->ShowCommand);
		LInstance->WaitForTermination = ASettings->ReadBool(
			ASection,
			"WaitForTermination",
			LInstance->WaitForTermination);
		Assign(LInstance);
	}
	catch(EProgramEntryError& LException) {
		throw EProgramEntryLoadError(LException.Message);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Save(TMemIniFile* ASettings, const AnsiString& ASection)
{
	try {
		ASettings->WriteString(
			ASection,
			"DisplayName",
			TAnsiStringUtilities::Quote(DisplayName));
		ASettings->WriteString(
			ASection,
			"File",
			TAnsiStringUtilities::Quote(File));
		ASettings->WriteString(
			ASection,
			"DocumentPrefix",
			TAnsiStringUtilities::Quote(DocumentPrefix));
		ASettings->WriteString(
			ASection,
			"DocumentSuffix",
			TAnsiStringUtilities::Quote(DocumentSuffix));
		ASettings->WriteString(
			ASection,
			"WorkingDirectory",
			TAnsiStringUtilities::Quote(WorkingDirectory));
		ASettings->WriteInteger(
			ASection,
			"ShowCommand",
			ShowCommand);
		ASettings->WriteBool(
			ASection,
			"WaitForTermination",
			WaitForTermination);
	}
	catch(EProgramEntryError& LException) {
		throw EProgramEntrySaveError(LException.Message);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntry::IsDefault()
{
	return File.Length() == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Initialize()
{
	try {
		FList = new TStringList();
		FList->CaseSensitive = true;
		FList->Sorted = true;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Uninitialize()
{
	if(FList != NULL) {
		for(int LI = 0; LI < FList->Count; ++LI) {
			delete FList->Objects[LI];
		}
		FList->Clear();
		delete FList;
		FList = NULL;
	}
}
//---------------------------------------------------------------------------
TProgramEntry* __fastcall TProgramEntriesModel::GetItem(int AIndex)
{
	return static_cast<TProgramEntry*>(List->Objects[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::GetCapacity()
{
	return List->Capacity;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::SetCapacity(int ACapacity)
{
	List->Capacity = ACapacity;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoInsertItems(const TIntegerDynArray& AIndexes)
{
	if(OnInsertItems != NULL) {
		OnInsertItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoDeleteItems(const TIntegerDynArray& AIndexes)
{
	if(OnDeleteItems != NULL) {
		OnDeleteItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoItemsChange()
{
	if(OnItemsChange != NULL) {
		OnItemsChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesModel::TProgramEntriesModel() :
	TModel(),
	FList(NULL),
	FOnInsertItems(NULL),
	FOnDeleteItems(NULL),
	FOnClear(NULL),
	FOnItemsChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesModel::~TProgramEntriesModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::AddItems(const DynamicArray<TProgramEntry*>& AItems)
{
	if(AItems.Length == 0) {
		return;
	}
	if(Capacity < Count + AItems.Length) {
		Capacity = Count + AItems.Length;
	}
	TAutoPointer<TInsertHistory> LHistory = new TInsertHistory();
	for(int LI = 0; LI < AItems.Length; ++LI) {
		int LIndex = List->AddObject(AItems[LI]->DisplayName, AItems[LI]);
		LHistory->Add(LIndex);
	}
	DoInsertItems(LHistory->Finish());
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	TAutoPointer<TDeleteHistory> LHistory = new TDeleteHistory();
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		int LIndex = AIndexes[LI];
		delete List->Objects[LIndex];
		List->Delete(LIndex);
		LHistory->Add(LIndex);
	}
	DoDeleteItems(LHistory->Finish());
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Clear()
{
	for(int LI = 0; LI < List->Count; ++LI) {
		delete List->Objects[LI];
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::ItemsChange()
{
	if(Count == 0) {
		return;
	}
	bool LSorted = List->Sorted;
	List->Sorted = false;
	try {
		for(int LI = 0; LI < List->Count; ++LI) {
			List->Strings[LI] = static_cast<TProgramEntry*>(List->Objects[LI])->DisplayName;
		}
	}
	catch(...) {
		List->Sorted = LSorted;
		throw;
	}
	List->Sorted = LSorted;
	DoItemsChange();
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::Find(TProgramEntry* AItem)
{
	return List->IndexOfObject(AItem);
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::FindByDisplayName(const AnsiString& ADisplayName)
{
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->DisplayName == ADisplayName) {
			return LI;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramEntriesModel::EnsureUniqueDisplayName(const AnsiString& ADisplayName)
{
	const int LMinIndex = 1;
	const int LMaxIndex = 100;
	AnsiString LDisplayName;
	for(int LIndex = LMinIndex; LIndex <= LMaxIndex; ++LIndex) {
		if(LIndex == LMinIndex) {
			LDisplayName = ADisplayName;
		}
		else {
			LDisplayName = AnsiString().sprintf("%s (%d)", ADisplayName.c_str(), LIndex);
		}
		if(FindByDisplayName(LDisplayName) >= 0) {
			if(LIndex == LMaxIndex) {
				throw EAppError("The program could not produce a unique display name.");
			}
			continue;
		}
		break;
	}
	return LDisplayName;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TProgramEntriesSelection::GetSelectionIndex()
{
	return ComboBox->ItemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::SetSelectionIndex(int ASelectionIndex)
{
	ComboBox->ItemIndex = ASelectionIndex;
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesSelection::TProgramEntriesSelection(TProgramEntriesModel* AProgramEntriesModel, TComboBox* AComboBox) :
	TSettingsHandler(),
	FProgramEntriesModel(AProgramEntriesModel),
	FComboBox(AComboBox)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesSelection::~TProgramEntriesSelection()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ComboBox->Items->BeginUpdate();
	try {
		for(int LI = 0; LI < AIndexes.Length; ++LI) {
			ComboBox->Items->InsertObject(
				AIndexes[LI],
				ProgramEntriesModel->Items[AIndexes[LI]]->DisplayName,
				ProgramEntriesModel->Items[AIndexes[LI]]);
		}
		ComboBox->ItemIndex = AIndexes[AIndexes.High];
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ComboBox->Items->BeginUpdate();
	try {
		int LItemIndex = ComboBox->ItemIndex;
		for(int LI = 0; LI < AIndexes.Length; ++LI) {
			ComboBox->Items->Delete(AIndexes[LI]);
			if(AIndexes[LI] < LItemIndex || LItemIndex == ProgramEntriesModel->Count) {
				--LItemIndex;
			}
		}
		if(ComboBox->Items->Count > 0 && LItemIndex >= 0) {
			ComboBox->ItemIndex = LItemIndex;
		}
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleClear(TObject* ASender)
{
	if(ComboBox->Items->Count == 0) {
		return;
	}
	ComboBox->Items->BeginUpdate();
	try {
		ComboBox->Items->Clear();
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleItemsChange(TObject* ASender)
{
	if(ProgramEntriesModel->Count == 0) {
		return;
	}
	ComboBox->Items->BeginUpdate();
	try {
		TProgramEntry* LProgramEntry = NULL;
		if(ComboBox->ItemIndex >= 0) {
			LProgramEntry = static_cast<TProgramEntry*>(ComboBox->Items->Objects[ComboBox->ItemIndex]);
		}
		for(int LI = 0; LI < ProgramEntriesModel->Count; ++LI) {
			ComboBox->Items->Strings[LI] = ProgramEntriesModel->Items[LI]->DisplayName;
			ComboBox->Items->Objects[LI] = ProgramEntriesModel->Items[LI];
		}
		if(LProgramEntry != NULL) {
			for(int LI = 0; LI < ProgramEntriesModel->Count; ++LI) {
				if(ProgramEntriesModel->Items[LI] == LProgramEntry) {
					ComboBox->ItemIndex = LI;
					break;
				}
			}
		}
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Initialize()
{
	try {
		DynamicArray<TProgramEntry*> LItems;
		LItems.Length = 1;
		{
			TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
			LItem->DisplayName = "Default - depends on file association";
			LItems[LItems.High] = LItem;
			LItem.Release();
		}
		ProgramEntriesModel->Clear();
		ProgramEntriesModel->AddItems(LItems);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesCommands::TProgramEntriesCommands(TProgramEntriesModel* AProgramEntriesModel, TProgramEntriesSelection* AProgramEntriesSelection) :
	TCommands(),
	FProgramEntriesModel(AProgramEntriesModel),
	FProgramEntriesSelection(AProgramEntriesSelection)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesCommands::~TProgramEntriesCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Add()
{
	if(!CanAdd()) {
		HandleFeatureUnavailable();
	}
	TProgramDialog::EnsureCreated();
	ProgramDialog->Caption = "Add";
	ProgramDialog->Reset();
	if(!ProgramDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
	LItem->Assign(ProgramDialog->Values);
	LItem->DisplayName = ProgramEntriesModel->EnsureUniqueDisplayName(LItem->DisplayName);
	ProgramEntriesModel->AddItems(TDynamicArrayUtilities<TProgramEntry*>::FromElements(LItem));
	LItem.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanAdd()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	if(Application->MessageBox(
		"Do you want to delete the program from the list?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	ProgramEntriesModel->DeleteItems(TDynamicArrayUtilities<int>::FromElements(ProgramEntriesSelection->SelectionIndex));
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanDelete()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Duplicate()
{
	TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
	LItem->Assign(ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]);
	LItem->DisplayName = ProgramEntriesModel->EnsureUniqueDisplayName(LItem->DisplayName);
	ProgramEntriesModel->AddItems(TDynamicArrayUtilities<TProgramEntry*>::FromElements(LItem));
	LItem.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanDuplicate()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Start()
{
	if(!CanStart()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->File;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not start the program.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanStart()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Explore()
{
	if(!CanExplore()) {
		HandleFeatureUnavailable();
	}
	AnsiString LDirectory = ExtractFileDir(ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->File);
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpVerb = "explore";
	LExecuteInfo.lpFile = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not explore the directory.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanExplore()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::ShowProperties()
{
	if(!CanShowProperties()) {
		HandleFeatureUnavailable();
	}
	TProgramEntry* LItem = ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex];
	TProgramDialog::EnsureCreated();
	ProgramDialog->Caption = "Properties";
	ProgramDialog->Values->Assign(LItem);
	if(!ProgramDialog->Execute()) {
		Abort();
	}
	if(ProgramDialog->Values->DisplayName != LItem->DisplayName) {
		ProgramDialog->Values->DisplayName = ProgramEntriesModel->EnsureUniqueDisplayName(ProgramDialog->Values->DisplayName);
	}
	LItem->Assign(ProgramDialog->Values);
	ProgramEntriesModel->ItemsChange();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanShowProperties()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".ProgramEntriesModel";
	if(ASettings->SectionExists(LSection)) {
		DynamicArray<TProgramEntry*> LItems;
		try {
			{
				LItems.Length = ASettings->ReadInteger(LSection, "Count", 0);
				for(int LI = 0; LI < LItems.Length; ++LI) {
					LItems[LI] = NULL;
				}
				for(int LI = 0; LI < LItems.Length; ++LI) {
					try {
						TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
						LItem->Load(ASettings, LSection + AnsiString().sprintf(".Item[%d]", LI));
						LItems[LI] = LItem;
						LItem.Release();
					}
					catch(EProgramEntryLoadError& LException) {
						Application->ShowException(&LException);
						LItems.Length = LItems.Length - 1;
						--LI;
					}
				}
			}
			ProgramEntriesModel->Clear();
			ProgramEntriesModel->AddItems(LItems);
		}
		catch(...) {
			for(int LI = 0; LI < LItems.Length; ++LI) {
				delete LItems[LI];
			}
			throw;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".ProgramEntriesModel";
	{
		ASettings->WriteInteger(LSection, "Count", ProgramEntriesModel->Count);
		for(int LI = 0; LI < ProgramEntriesModel->Count; ++LI) {
			ProgramEntriesModel->Items[LI]->Save(ASettings, LSection + AnsiString().sprintf(".Item[%d]", LI));
		}
	}
}
//---------------------------------------------------------------------------
} // End namespace App_programs
//---------------------------------------------------------------------------

